<html>
  <head>
    <script language="javascript">
      <!--
          window.location="{{ redirect_url }}";
      //-->
    </script>
  </head>
  <body>
    <noscript>
      <meta http-equiv="refresh" content="1;url={{ redirect_url }}">
    </noscript>
  </body>
</html>
